import 'dart:io';

import 'package:camera/camera.dart';
import 'package:rc_fl/fl_logger.dart';
import 'package:rc_fl/model/api_properties.dart';
import 'package:rc_fl/model/fl_properties.dart';
import 'package:rc_fl/model/logger_properties.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:google_ml_kit/google_ml_kit.dart';
import 'package:yaml/yaml.dart';

import 'face_service_import.dart';

/// @author: ygao
/// @date: 2021/6/21
/// @version: 1.0
/// @description:
class ApplicationContext {
  ApplicationContext._();

  static ApplicationContext? _instance;

  static ApplicationContext get instance => _getInstance();

  static ApplicationContext _getInstance() {
    if (_instance == null) {
      _instance = ApplicationContext._();
    }
    return _instance!;
  }

  Map beanMap = Map();
  late YamlMap yamlMap;
  late Map variableMap;
  late String privacy;
  List<CameraDescription> cameras = [];

  FaceDetector faceDetector = GoogleMlKit.vision.faceDetector(FaceDetectorOptions(
    enableContours: true,
    enableClassification: true,
  ));

  init() async {
    await this.loadApplication();
    this.loadPrivacy();
    FaceService().loadModel();
    FlLogger.info(1111);
    FlLogger.info(kIsWeb);
    // FlLogger.info(Platform.isAndroid);
    if (!kIsWeb&&Platform.isAndroid) {
      cameras = await availableCameras();
    }
  }

  MapEntry convertVariable(key, value) {
    var match = RegExp(r'\$\{(.*)\}').firstMatch(value.toString());
    if (match != null) {
      var value2 = variableMap[match.group(1)];
      return MapEntry(key, value2);
    }
    return MapEntry(key, value);
  }

  parseFlProperties() async {
    YamlMap ym = yamlMap.nodes['fl']!.value;
    Map profiles = ym['profiles'].value;
    String? profilesActive = profiles['active'];
    if (profilesActive != null) {
      var profilesStr = await rootBundle.loadString('config/application-$profilesActive.yaml');
      variableMap = await loadYaml(profilesStr);
      print('profile-$profilesActive');
      print(variableMap);
    }

    FlProperties flProperties = FlProperties();
    flProperties.loggerProperties = LoggerProperties.fromMap(ym['logger'].value.map(convertVariable));
    flProperties.apiProperties = ApiProperties.fromMap(ym['api'].value.map(convertVariable));
    addBean('flProperties', flProperties);
  }

  loadApplication() async {
    var yamlStr = await rootBundle.loadString('config/application.yaml');
    yamlMap = loadYaml(yamlStr);
    print("application:");
    print(yamlMap.nodes);
    await this.parseFlProperties();
  }

  loadPrivacy() async {
    this.privacy = await rootBundle.loadString('PRIVACY');
    FlLogger.debug(this.privacy);
  }

  addBean(String key, object) {
    beanMap[key] = object;
  }

  getBean(String key) {
    return beanMap[key];
  }
}
