/// @author: ygao
/// @date: 2023/4/21
/// @version: 1.0
/// @description:

import 'package:rc_fl/common/common_types.dart';
import 'package:flutter/material.dart';

class FlListView extends StatefulWidget {
  final int? count;
  final IntWidgetFunction getCell;
  final VoidCallback? loadMore;
  final RefreshCallback? onRefresh;
  final FlListViewType flListViewType;
  final AppBar? appBar;

  FlListView({
    Key? key,
    this.count,
    required this.getCell,
    this.loadMore,
    this.flListViewType = FlListViewType.column,
    this.onRefresh,
    this.appBar,
  }) : super(key: key);

  @override
  FlListViewState createState() => FlListViewState();
}

class FlListViewState extends State<FlListView> {
  ScrollController controller = new ScrollController();
  bool toTopButtonVisible = false;

  @override
  void initState() {
    controller.addListener(() {
      double topLimit = 500;
      if (controller.offset < topLimit && toTopButtonVisible) {
        toTopButtonVisible = false;
        setState(() {});
      } else if (controller.offset > topLimit && !toTopButtonVisible) {
        toTopButtonVisible = true;
        setState(() {});
      }
      if (widget.loadMore != null && controller.position.maxScrollExtent == controller.position.pixels) {
        widget.loadMore!();
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    var listView;
    if (widget.flListViewType == FlListViewType.column) {
      listView = ListView(
        controller: controller,
        children: List.generate(
          widget.count!,
          ((int index) => widget.getCell(index)),
        ),
      );
    } else {
      double width = MediaQuery.of(context).size.width;
      int columnCount = width ~/ 500 + 1;
      int rowConunt = widget.count! ~/ columnCount + 1;
      listView = ListView(
        controller: controller,
        children: [
          Column(
            children: [
              ...List<Widget>.generate(
                rowConunt,
                (y) {
                  return Row(
                    children: [
                      ...List<Widget>.generate(columnCount, (x) {
                        int index = columnCount * y + x;
                        var card = Padding(padding: EdgeInsets.all(10), child: (index > widget.count! - 1) ? Container() : widget.getCell(index));
                        return Expanded(
                          child: card,
                        );
                      }),
                    ],
                  );
                },
              ),
            ],
          )
        ],
      );
    }
    var result = Scaffold(
      appBar: widget.appBar,
      body: RefreshIndicator(
        child: listView,
        onRefresh: widget.onRefresh ?? () async {},
      ),
      floatingActionButton: !toTopButtonVisible
          ? null
          : FloatingActionButton(
              child: Icon(Icons.arrow_upward),
              onPressed: () {
                controller.animateTo(
                  .0,
                  duration: Duration(milliseconds: 200),
                  curve: Curves.ease,
                );
              },
            ),
    );
    return result;
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }
}

enum FlListViewType {
  column,
  wrap,
}
